#-------------------------------------------------------------------------------
# Name:        SimpleEthernet
# Purpose:     A very trimmed down set of classes to unpack the common network packet formats
#
# Author:      DCollins
#
# Created:     19/12/2013
#
# Copyright 2014 Diarmuid Collins
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License
#    as published by the Free Software Foundation; either version 2
#    of the License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


import struct
import socket
import array

def unpack48(x):
    '''Convert a 48 bit buffer into an integer'''
    x2, x3 = struct.unpack('>HI', x)
    return x3 | (x2 << 32)



def mactoreadable(macaddress):
    '''Convert a macaddress into the readable form'''
    mac_string = ""
    for i in range(5,0,-1):
        eachbyte = 0xff & int(macaddress >> (i*8))
        mac_string += ":{:02x}".format(eachbyte)
    return mac_string


def calc_checksum(pkt):
    '''Calculate the checksum of a packet'''
    if len(pkt) % 2 == 1:
        pkt += "\0"
    s = sum(array.array("H", pkt))
    s = (s >> 16) + (s & 0xffff)
    s += s >> 16
    s = ~s
    return s & 0xffff


class Ethernet(object):
    '''This is simple class to build or deconstruct an Ethernet packet'''
    HEADERLEN = 14
    TYPE_IP = 0x800

    def __init__(self,buf=None):
        '''Constructor for and Ethernet packet'''
        self.type = None
        """:type : int"""
        self.srcmac = None
        """:type : int"""
        self.dstmac = None
        """:type : int"""
        self.payload = None
        """:type : str"""

        if buf != None:
            self.unpack(buf)

    def unpack(self,buf):
        '''Unpack a buffer into an Ethernet object
        :type buf: str
        '''
        self.dstmac = unpack48(buf[:6])
        self.srcmac = unpack48(buf[6:12])
        (self.type,) = struct.unpack_from('>H',buf,12)
        self.payload = buf[Ethernet.HEADERLEN:]


    def pack(self):
        '''Pack the Ethernet object into a bufferr
        :rtype : str
        '''
        if self.dstmac == None or self.srcmac == None or self.type == None or self.payload == None:
            raise ValueError("All thre required Ethernet fields are not complete")
        header = struct.pack('>HIHIH',(self.dstmac>>32),(self.dstmac&0xffffffff),(self.srcmac>>32),(self.srcmac&0xffffffff),0x0800)
        return header + self.payload



class IP():
    '''Create or unpack an IP packet'''
    PROTOCOLS = {"ICMP":0x01,"IGMP" : 0X02, "TCP":0x6,"UDP":0x11}
    IP_HEADER_FORMAT = '>BBHHBBBBHII'
    IP_HEADER_SIZE = struct.calcsize(IP_HEADER_FORMAT)

    def __init__(self,buf=None):
        self.srcip = None
        """:type str"""
        self.dstip = None
        """:type str"""
        self.len = None
        """:type int"""
        self.flags = 0x0
        """:type int"""
        self.protocol = IP.PROTOCOLS['UDP'] # default to udp
        """:type int"""
        self.payload = None
        """:type str"""
        self.version = 4 # IPV4
        """:type int"""
        self.ihl = 5 # Header len in 32 bit words
        """:type int"""
        self.dscp = 0
        """:type int"""
        self.id = 0
        """:type int"""
        self.ttl = 20
        """:type int"""
        if buf != None:
            self.unpack(buf)

    def unpack(self,buf):
        '''Unpack a buffer into an ethernet object'''

        if len(buf) < IP.IP_HEADER_SIZE:
            raise ValueError("Buffer too short for to be an IP packet")
        (na1,self.dscp, self.len,self.id,self.flags,na3, self.ttl, self.protocol, checksum, self.srcip,self.dstip) = struct.unpack_from(IP.IP_HEADER_FORMAT,buf)
        self.flags = self.flags >> 5
        #self.version = na1 >>4
        #self.ihl = na1 & 0xf
        self.srcip = socket.inet_ntoa(struct.pack('!I',self.srcip))
        self.dstip = socket.inet_ntoa(struct.pack('!I',self.dstip))
        self.payload = buf[IP.IP_HEADER_SIZE:self.len]

    def pack(self):
        '''Pack the IP object into a string buffer
        :rtype :str
        '''
        for word in [self.dscp,self.id,self.flags,self.ttl,self.protocol,self.srcip,self.dstip]:
            if word == None:
                raise ValueError("All required IP payloads not defined")

        (srcip_as_int,) = struct.unpack('!I',socket.inet_aton(self.srcip))
        (dstip_as_int,) = struct.unpack('!I',socket.inet_aton(self.dstip))
        self.len = IP.IP_HEADER_SIZE+len(self.payload)
        header = struct.pack(IP.IP_HEADER_FORMAT,0x45,self.dscp,self.len,self.id,self.flags,0,self.ttl,self.protocol,0,srcip_as_int,dstip_as_int)
        checksum = calc_checksum(header)
        header = header[:10] + struct.pack('H',checksum) + header[12:]
        return header + self.payload


class UDP():
    '''Class to build and unpack a UDP packet'''
    UDP_HEADER_FORMAT = '>HHHH'
    UDP_HEADER_SIZE = struct.calcsize(UDP_HEADER_FORMAT)
    def __init__(self,buf=None):

        self.srcport = None
        self.dstport = None
        self.len = None
        self.payload = None

        if buf != None:
            self.unpack(buf)

    def unpack(self,buf):
        '''Unpack a buffer into a UDP object'''

        if len(buf) < UDP.UDP_HEADER_SIZE:
            raise ValueError("Buffer too short to be a UDP packet")
        (self.srcport,self.dstport,self.len,checksum) = struct.unpack_from(UDP.UDP_HEADER_FORMAT,buf)
        self.payload = buf[UDP.UDP_HEADER_SIZE:]

    def pack(self):
        '''Pack a UDP object into a buffer
        :rtype :str
        '''
        if self.srcport == None or self.dstport == None or self.payload == None:
            raise ValueError("All UDP fields need to be defined to pack the payload")

        self.len = len(self.payload) + UDP.UDP_HEADER_SIZE
        return struct.pack(UDP.UDP_HEADER_FORMAT,self.srcport,self.dstport,self.len,0) + self.payload


class AFDX():
    '''This class will  unpack an AFDX packet'''
    HEADERLEN = 14
    DSTMAC_CONST = 0x3000000
    SRCMAC_CONST = 0x20000
    MIN_PAYLOAD_LEN = 42
    def __init__(self,buf=None):
        self.type =None
        self.networkID = None
        self.equipmentID = None
        self.interfaceID = None
        self.vlink = None

        self.payload = None
        self.sequencenum = None
        if buf != None:
            self.unpack(buf)

    def unpack(self,buf):
        self.set_dstmac(buf[:6])
        self.unpacksrcmac(unpack48(buf[6:12]))

        (self.type,) = struct.unpack_from('>H',buf,12)
        self.payload = buf[AFDX.HEADERLEN:-1]
        self.sequencenum = struct.unpack('B',buf[-1])

    def unpacksrcmac(self,mac):
        srcconstantf = mac >> 24
        #if srcconstantf != AFDX.SRCMAC_CONST:
        #    raise ValueError('Expected constant field of {:#x} in SrcMac Address'.format(AFDX.SRCMAC_CONST))
        #(self.networkID,self.equipmentID,self.interfaceID) = struct.unpack_from('BBB',mac[:3])
        #self.interfaceID = self.interfaceID >> 5

    def set_dstmac(self,mac):
        (dstconstantf,vlink) = struct.unpack_from('>IH',mac)
        #if dstconstantf != AFDX.DSTMAC_CONST:
        #    raise ValueError('Expected constant field of {:#x} in DestMac Address'.format(AFDX.DSTMAC_CONST))
        self.vlink = vlink

    def pack(self):

        if (len(self.payload) < AFDX.MIN_PAYLOAD_LEN):
            raise ValueError('Minimum Payload of {} bytes'.format(AFDX.MIN_PAYLOAD_LEN))

        afdx_header = struct.pack('>IHHBBBBH',AFDX.DSTMAC_CONST,self.vlink,(AFDX.SRCMAC_CONST>>8),0,self.networkID,self.equipmentID,(self.interfaceID<<5),self.type)
        packet = afdx_header + self.payload + struct.pack('>B',self.sequencenum)

        return packet





#-------------------------------------------------------------------------------
# Name:        IENA
# Purpose:     Class to pack and unpack IENA packets
#
# Author:      DCollins
#
# Created:     19/12/2013
#
# Copyright 2014 Diarmuid Collins
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License
#    as published by the Free Software Foundation; either version 2
#    of the License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import struct
import datetime,time

def unpack48(x):
    x2, x3 = struct.unpack('>HI', x)
    return x3 | (x2 << 32)


class IENA ():
    '''Class to pack and unpack IENA(tm) payloads. IENA(tm) is an proprietary payload format
    developed by Airbus for use in FTI networks. It is usually transmitted in a UDP packet
     containing parameter data acquired from sensors and buses'''
    IENA_HEADER_FORMAT = '>HHHIBBH'
    IENA_HEADER_LENGTH = struct.calcsize(IENA_HEADER_FORMAT)
    TRAILER_LENGTH = 2


    def __init__(self):
        '''Constructor class for an IENA payload'''


        self.key = None # know as ienaky
        """:type : int"""
        self.size = None
        """:type : int"""
        self.timeusec = None
        """:type : int"""
        self.keystatus = None
        """:type : int"""
        self.status = None
        """:type : int"""
        self.sequence = None
        """:type : int"""
        self.endfield = 0xdead
        """:type : int"""
        self.payload = None #string containing payload
        """:type : str"""

        self._packetStrut = struct.Struct(IENA.IENA_HEADER_FORMAT)
        # only calculate this once
        self._startOfYear = datetime.datetime(datetime.datetime.today().year, 1, 1, 0, 0, 0,0)
        self.lengthError = False # Flag to verify the buffer length





    def unpack(self,buf,ExceptionOnLengthError=False):
        '''Unpack a raw byte stream to an IENA object
        Accepts a buffer to unpack as the required argument
        :type buf: str
        :type ExceptionOnLengthError: bool
        '''
        # Some checking
        if len(buf) < IENA.IENA_HEADER_LENGTH:
            raise ValueError("Buffer passed to unpack is too small to be an IENA packet")

        (self.key, self.size, timehi, timelo, self.keystatus, self.status, self.sequence)  = self._packetStrut.unpack_from(buf)
        self.timeusec = timelo + timehi * 2**32

        if self.size*2 != len(buf):
            self.lengthError = True
            if ExceptionOnLengthError:
                raise ValueError

        self.payload = buf[IENA.IENA_HEADER_LENGTH:-2]
        (self.endfield,) = struct.unpack(">H",buf[-2:]) # last two bytes are the trailer


    def pack(self):
        '''Pack the IENA payload into a binary format
        :rtype: str
        '''
        timehi = self.timeusec >> 32
        timelo = self.timeusec % 0x100000000
        for required_field in [self.key,timehi,timelo,self.keystatus,self.status,self.sequence,self.endfield,self.payload]:
            if required_field == None:
                raise ValueError("A required field in the IENA packet is not defined")

        self.size =  (len(self.payload)  + IENA.IENA_HEADER_LENGTH + IENA.TRAILER_LENGTH) /2 # size is in words

        packetvalues = (self.key,self.size,timehi,timelo,self.keystatus,self.status,self.sequence)
        packet = self._packetStrut.pack(*packetvalues) + self.payload + struct.pack('>H',self.endfield)
        return packet


    def _getPacketTime(self):
        '''Return the Packet time in standard UNIX time
        :rtype: int
        '''
        return int(self.timeusec/1e6 + time.mktime(self._startOfYear.timetuple()))

    def setPacketTime(self,utctimestamp,microseconds=0):
        ''''Set the packet timestamp
        :type timestamp: int
        :type microseconds: int
        '''
        seconds_this_year = utctimestamp - int(time.mktime(self._startOfYear.timetuple()))
        packettime = microseconds + int(seconds_this_year)*1000000
        self.timeusec = packettime










#-------------------------------------------------------------------------------
# Name:        ParserAlignedPacket
# Purpose:
#
# Author:      DCollins
#
# Created:     19/12/2013
#
# Copyright 2014 Diarmuid Collins
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License
#    as published by the Free Software Foundation; either version 2
#    of the License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import struct


class ParserAlignedPacket():
    """A class that handles parser aligned packets. Unpack a buffer to populate the field into a list of parserblocks"""
    def __init__(self):
        self.parserblocks = []
        self.numberofblocks =0


    def unpack(self,buf):
        """Pass a buffer containing all the Parser alignd payload and this method will
        unpack all the fields. Returns a list of parserblocks"""
        fullbufferlen = len(buf)
        bufferparsed = 0
        while bufferparsed < fullbufferlen:
            block = ParserAlignedBlock()
            # unpack and add the amount unpacked to the running total
            bufferparsed += block.unpack(buf[bufferparsed:])
            self.parserblocks.append(block)



class ParserAlignedBlock():
    """A class to handle a single Parser Block. Returns an object containing all the fields of a parser block"""
    def __init__(self):

        self.error = False
        self.errorcode = 0
        self.quadbytes = 1
        self.messagecount = 0
        self.busid = 1
        self.elapsedtime = 2
        self.payload = ""

        self.format = '>HBBL'
        self.headerlen = struct.calcsize(self.format)


    def unpack(self,buf):
        """Unpack a single parser block. Unsually called only from the iNetXParser unpack method. Returns the length of the
        parser block"""
        # Unpack the fields
        (error_and_quad,self.messagecount,self.busid,self.elapsedtime)=struct.unpack_from(self.format,buf)
        # MSB bit is the error flag
        if ((error_and_quad >> 15) == 1):
            self.error = True
        else:
            self.error = False

        # As error code and quad bytes are not byte aligned I have to do some shifting and masking
        self.errorcode = (error_and_quad >> 9) & 0x3f
        self.quadbytes = error_and_quad & 0x1ff
        payload_length_in_bytes = (self.quadbytes - 2)*4

        # Raise an exception if the payload is not as expected
        if len(buf) < self.headerlen+payload_length_in_bytes:
            raise ValueError
        self.payload = buf[self.headerlen:self.quadbytes*4]
        # Return the length of the block as a courtesy
        return self.quadbytes*4


class ARINC429():
    """This is not working yet. Don't use it"""
    MESSAGE_LEN = 4 # 4 bytes
    LABEL_REVERSE = [
        0x00, 0x80, 0x40, 0xc0, 0x20, 0xa0, 0x60, 0xe0, #   0 -   8
        0x10, 0x90, 0x50, 0xd0, 0x30, 0xb0, 0x70, 0xf0, #   9 -  15
        0x08, 0x88, 0x48, 0xc8, 0x28, 0xa8, 0x68, 0xe8, #  16 -  23
        0x18, 0x98, 0x58, 0xd8, 0x38, 0xb8, 0x78, 0xf8, #  24 -  31
        0x04, 0x84, 0x44, 0xc4, 0x24, 0xa4, 0x64, 0xe4, #  32 -  39
        0x14, 0x94, 0x54, 0xd4, 0x34, 0xb4, 0x74, 0xf4, #  40 -  47
        0x0c, 0x8c, 0x4c, 0xcc, 0x2c, 0xac, 0x6c, 0xec, #  48 -  55
        0x1c, 0x9c, 0x5c, 0xdc, 0x3c, 0xbc, 0x7c, 0xfc, #  56 -  63
        0x02, 0x82, 0x42, 0xc2, 0x22, 0xa2, 0x62, 0xe2, #  64 -  71
        0x12, 0x92, 0x52, 0xd2, 0x32, 0xb2, 0x72, 0xf2, #  72 -  79
        0x0a, 0x8a, 0x4a, 0xca, 0x2a, 0xaa, 0x6a, 0xea, #  80 -  87
        0x1a, 0x9a, 0x5a, 0xda, 0x3a, 0xba, 0x7a, 0xfa, #  88 -  95
        0x06, 0x86, 0x46, 0xc6, 0x26, 0xa6, 0x66, 0xe6, #  96 - 103
        0x16, 0x96, 0x56, 0xd6, 0x36, 0xb6, 0x76, 0xf6, # 104 - 111
        0x0e, 0x8e, 0x4e, 0xce, 0x2e, 0xae, 0x6e, 0xee, # 112 - 119
        0x1e, 0x9e, 0x5e, 0xde, 0x3e, 0xbe, 0x7e, 0xfe, # 120 - 127
        0x01, 0x81, 0x41, 0xc1, 0x21, 0xa1, 0x61, 0xe1, # 128 - 135
        0x11, 0x91, 0x51, 0xd1, 0x31, 0xb1, 0x71, 0xf1, # 136 - 143
        0x09, 0x89, 0x49, 0xc9, 0x29, 0xa9, 0x69, 0xe9, # 144 - 151
        0x19, 0x99, 0x59, 0xd9, 0x39, 0xb9, 0x79, 0xf9, # 152 - 159
        0x05, 0x85, 0x45, 0xc5, 0x25, 0xa5, 0x65, 0xe5, # 160 - 167
        0x15, 0x95, 0x55, 0xd5, 0x35, 0xb5, 0x75, 0xf5, # 168 - 175
        0x0d, 0x8d, 0x4d, 0xcd, 0x2d, 0xad, 0x6d, 0xed, # 176 - 183
        0x1d, 0x9d, 0x5d, 0xdd, 0x3d, 0xbd, 0x7d, 0xfd, # 184 - 191
        0x03, 0x83, 0x43, 0xc3, 0x23, 0xa3, 0x63, 0xe3, # 192 - 199
        0x13, 0x93, 0x53, 0xd3, 0x33, 0xb3, 0x73, 0xf3, # 200 - 207
        0x0b, 0x8b, 0x4b, 0xcb, 0x2b, 0xab, 0x6b, 0xeb, # 208 - 215
        0x1b, 0x9b, 0x5b, 0xdb, 0x3b, 0xbb, 0x7b, 0xfb, # 216 - 223
        0x07, 0x87, 0x47, 0xc7, 0x27, 0xa7, 0x67, 0xe7, # 224 - 231
        0x17, 0x97, 0x57, 0xd7, 0x37, 0xb7, 0x77, 0xf7, # 232 - 239
        0x0f, 0x8f, 0x4f, 0xcf, 0x2f, 0xaf, 0x6f, 0xef, # 240 - 247
        0x1f, 0x9f, 0x5f, 0xdf, 0x3f, 0xbf, 0x7f, 0xff, # 248 - 255
        0x00
    ]
    def __init__(self):
        self.parity = None
        self.ssm = None
        self.data = None
        self.sdi = None
        self.label = None

    def unpack(self,buf):
        if len(buf) != ARINC429.MESSAGE_LEN:
            raise ValueError
        (byte1,byte2,byte3,byte4) = struct.unpack('BBBB',buf)
        self.parity = byte1 / 128
        self.ssm = (byte1/32) % 4
        self.data = ((byte1 %32) * 256 + byte2 ) * 64 + (byte3 /4)
        self.sdi = byte3 % 4
        self.label = ARINC429.LABEL_REVERSE[byte4]






#-------------------------------------------------------------------------------
# Name:        iNetX
# Purpose:     Class to construct and de construct iNetx Packets
#
# Author:      DCollins
#
# Created:     19/12/2013
#
# Copyright 2014 Diarmuid Collins
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License
#    as published by the Free Software Foundation; either version 2
#    of the License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import struct


class iNetX ():
    '''Class to pack and unpack iNetX payloads. iNet-X is an open payload format for use
    in FTI networks. It is usually transmitted in a UDP packet containing parameter data
    acquired from sensors and buses'''
    DEF_CONTROL_WORD = 0x11000000
    INETX_HEADER_FORMAT = '>LLLLLLL'
    INETX_HEADER_LENGTH = struct.calcsize(INETX_HEADER_FORMAT)


    def __init__(self):
        '''Creator method for an iNetX class'''
        self.inetxcontrol = iNetX.DEF_CONTROL_WORD
        """:type : int"""
        self.streamid = None
        """:type : int"""
        self.sequence = None
        """:type : int"""
        self.packetlen = None
        """:type : int"""
        self.ptptimeseconds = None
        """:type : int"""
        self.ptptimenanoseconds = None
        """:type : int"""
        self.pif = None
        """:type : int"""
        self.payload = None
        """:type : str"""

        self._packetStrut = struct.Struct(iNetX.INETX_HEADER_FORMAT)



    def pack(self):
        '''Pack the packet into a binary format and return as a string
        :rtype: str
        '''
        for required_field in (self.inetxcontrol,self.streamid,self.sequence,self.ptptimeseconds,self.ptptimenanoseconds,self.pif,self.payload):
            if required_field == None:
                raise ValueError("A required field in the iNet-X packet is not defined")

        self.packetlen =  len(self.payload)  + iNetX.INETX_HEADER_LENGTH
        packetvalues = (self.inetxcontrol,self.streamid,self.sequence,self.packetlen,self.ptptimeseconds,self.ptptimenanoseconds,self.pif )
        packet = self._packetStrut.pack(*packetvalues) + self.payload
        return packet


    def unpack(self,buf):
        '''
        Unpack a raw byte stream to an iNetX object.
        Accepts a buffer to unpack as the required argument
        :type buf: str
        :type checkcontrol: bool
        '''
        if len(buf) < iNetX.INETX_HEADER_LENGTH:
            raise ValueError ("Buffer is too short to be an iNetX packet")

        self.inetxcontrol,self.streamid,self.sequence,self.packetlen,self.ptptimeseconds,self.ptptimenanoseconds,self.pif  = self._packetStrut.unpack_from(buf)

        if self.packetlen != len(buf):
            raise ValueError("Length of buffer 0x{:X} does not match length field 0x{:X}".format(len(buf),self.packetlen))

        self.payload = buf[iNetX.INETX_HEADER_LENGTH:]


    def setPacketTime(self,utctimestamp,nanoseconds=0):
        ''''Set the packet timestamp
        :type timestamp: int
        :type nanoseconds: int
        '''
        self.ptptimeseconds = utctimestamp
        self.ptptimenanoseconds = nanoseconds











#-------------------------------------------------------------------------------
# Name:        iNetXStream
# Purpose:
#
# Author:      DCollins
#
# Created:     19/12/2013
# Copyright:   (c) DCollins 2013
# Licence:     <your licence>
#-------------------------------------------------------------------------------



#******************************************************
#
# DO NOT USE THIS CLASS. I WILL REMOVE IT BUT NOT AT THE MOMENT
#
#*****************************************************





import inetx

def getaccuratetiming():
    if sys.platform.startswith('linux'):
        return time.time()
    else:
        return time.clock()


# Install the following:

#pypcap : http://code.google.com/p/pypcap/issues/detail?id=11#c10
#dpkt : http://code.google.com/p/dpkt/downloads/list
#numpy: http://sourceforge.net/projects/numpy/files/NumPy/1.6.1/

# If you want to capture from the network
#Zope: Download the egg file http://pypi.python.org/pypi/zope.interface/3.8.0#downloads and then install it using easy_install
#      eg: python ./site-packages/easy_install.py zope.interface-3.8.0-py2.7-win32.egg
#Twisted: http://twistedmatrix.com/trac/wiki/Downloads

class iNetXStream(socket.socket):
    '''Class describing a stream if iNetX packets. When you init this it creates a load of defaults
    You can then modify it to your hearts content'''
    def __init__(self,length=10,payloadsize=1000,parserblocks=0,proto="UDP"):

        self.protocol = proto
        if self.protocol =="UDP":
            self._sock = socket.socket(
                socket.AF_INET,
                socket.SOCK_DGRAM,
                socket.IPPROTO_UDP)

        else:
            self._sock = socket.socket(
                socket.AF_INET,
                socket.SOCK_STREAM)

        self.length = length # the length of the stream in packets
        self.parserblocks = parserblocks # The number of parser blocks
        self.payloadsize = payloadsize # if we set the parserblocks to zero then random data is generated
        self.packets = [] # list of packets
        self.mbitrate=0
        self._delayloop=5000
        self._delayloopres=0


        # The udp/ip info
        self.dstip = "192.168.28.1"
        self.dstport = 8181
        self.srcport = 1616
        self.hostip = "192.168.28.100"

        # validation stuff
        self.missedseqences = defaultdict(list)
        self.missedcount = 0
        self.missedtimes = []
        self.missedpacket = defaultdict(list)
        self.streamsize = 0
        self.packetschecked = 0
        self.firsttimestamp = sys.float_info.max
        self.lasttimestamp = 0.0

        self._loadmbitdly()





    def buildstream(self):
        '''This builds a stream of xnet packets. It's done in memory and then dumped down the
        ethernet socket'''
        # Open the socket then fire down the packets in chunks

        for i in range(self.length):

            packet = iNetX() # create a iNetX packet
            packet.sequence = i

            # fill the payload
            if self.parserblocks > 0:
                # we want to create a parser aligned payload
                packet.parserPayload(self.parserblocks,quadbytes=3,count=i*self.parserblocks)
            else:
                packet.randomPayload(self.payloadsize)

            # build it
            packet.buildpacket()
            self.packets.append(packet)



    def sendstream(self):
        '''Generates a stream of iNetX packets'''
        # Open the socket then fire down the packets in chunks

        self.buildstream()

        self._sock.bind((self.hostip,self.srcport))
#        prev_time = time.clock()
#        if self.mbitrate > 70:
#            sleeptime = 0
#        elif self.mbitrate > 40:
#            sleeptime = 0.0002
#        else:
#            sleeptime = 0.0004

        i=0
        for packet in self.packets:
            # adjust the bandwidth if we defined the mbitrate
#            if ( i%30 == 29 and self.mbitrate > 0):
#                now_time = time.clock()
#                delta = now_time-prev_time
#                mbitrate = int(packet.bytes * 8 * 29/(1000000*delta)) # we have had 100 packets
#                if (mbitrate > self.mbitrate):
#                    sleeptime += 0.0002
#                else:
#                    sleeptime = 0
#                prev_time = time.clock()
#            i += 1
#            if sleeptime > 0:
#                time.sleep(sleeptime)


            self._sock.sendto(packet.packet,(self.dstip,self.dstport))


    def buildandsend(self,pkttype="inetx",bench=False):
        '''This combines the other two methods. It generates the packets at the same time
        as it dumps it onto the interface. This ensures that you can generate a constants stream
        of data. I get about 65Mbit/s at a max rate'''
        self._sock.bind((self.hostip,self.srcport))
        start_time_of_100_packets = time.clock()
        current_time = time.time()
        ptpsec  = int(current_time)
        ptpnsec = int(math.modf(current_time)[0] * 1e9)
        mbitrate = 0


        loopdelay = 1000
        if mbitrate < 1:
            loopdelay = self.mbitrate
        elif mbitrate < 10:
            loopdelay = 10*self.mbitrate
        elif mbitrate < 100:
            loopdelay = 100

        for i in range(self.length):

            if pkttype == "inetx":
                packet = iNetX()
                packet.streamid = self.mbitrate
                packet.sequence = i % pow(2,32)
                packet.ptptimeseconds = ptpsec
                packet.ptptimenanoseconds = ptpnsec
            else:
                packet = IENA()
                packet.sequence = i % pow(2,16)


            # fill the payload
            if self.parserblocks > 0:
                # we want to create a parser aligned payload
                packet.parserPayload(self.parserblocks,quadbytes=3,count=i*self.parserblocks)
            else:
                packet.randomPayload(self.payloadsize)

            # build it
            packet.pack()
            self.streamsize += packet.bytes
            for j in range(self._delayloop):
                self._delayloopres += math.sqrt(j+self._delayloopres)
            self._delayloopres = 0
            if self.protocol == "UDP":
                self._sock.sendto(packet.packet,(self.dstip,self.dstport))
            else:
                self._sock.connect((self.dstip,self.dstport))
                self._sock.sendall(packet.packet)


            if bench == False:
                prev_mbitrate =  0
                # keep an eye on the data rate
                if i % loopdelay == (loopdelay-1):
                    end_time_of_100_packets = getaccuratetiming()
                    time_since_start_of_100_packets = end_time_of_100_packets - start_time_of_100_packets
                    bytes_sent = packet.bytes * (loopdelay-1)
                    prev_mbitrate = mbitrate
                    mbitrate =  int((8*bytes_sent)/(time_since_start_of_100_packets*1000*1000))

                    # Trying to figure out a wway to get to the target rate fast
                    if mbitrate < 5 :
                        scaling = 100
                    elif mbitrate < 10:
                        scaling = 50
                    elif mbitrate < 30:
                        scaling = 1
                    else:
                        scaling = 1

                    if abs(mbitrate - self.mbitrate ) < 5:
                        dlyloop_change = 1*scaling
                    elif abs(mbitrate - self.mbitrate) < 10:
                        dlyloop_change = 5*scaling
                    elif abs(mbitrate - self.mbitrate) < 15:
                        dlyloop_change = 10*scaling
                    else:
                        dlyloop_change = 20*scaling
                    if dlyloop_change > 5000:
                        dlyloop_change = 5000

                    if mbitrate > self.mbitrate:
                        self._delayloop = self._delayloop + dlyloop_change;
                    elif mbitrate < self.mbitrate:
                        self._delayloop = self._delayloop - dlyloop_change;
                    start_time_of_100_packets = getaccuratetiming()
                    # update the ptptimestamp
                    current_time = time.time()
                    ptpsec  = int(current_time)
                    ptpnsec = int(math.modf(current_time)[0] * 1e9)

                if i % (loopdelay*10) == loopdelay*10-1:
                    print ("INFO Current bit rate = {} Mbit/s. Delay loop = {} Data sent = {} MB Delay loop change = {} ".format(mbitrate,self._delayloop,self.streamsize/1000000,dlyloop_change))

    def sendudp(self,bench=False):
        '''This proc just sends a stream of upd packets. Need to move this out of this class as it's not a inetx stream'''
        self._sock.bind(('',self.srcport))
        start_time_of_100_packets = time.clock()
        for i in range(self.length):

            #payload = ''.join(['\x05' for num in xrange(self.payloadsize)])
            payload = '\x05' * self.payloadsize
            payloadsize = self.payloadsize + 32
            self.streamsize += payloadsize
            for j in range(self._delayloop):
                self._delayloopres += math.sqrt(j+self._delayloopres)
            self._delayloopres = 0
            self._sock.sendto(payload,(self.dstip,self.dstport))

            if bench == False:
                # keep an eye on the data rate
                if i % 1000 == 999:
                    end_time_of_100_packets = time.clock()
                    time_since_start_of_100_packets = end_time_of_100_packets - start_time_of_100_packets
                    bytes_sent = payloadsize * 999
                    mbitrate =  int((8*bytes_sent)/(time_since_start_of_100_packets*1000*1000))
                    if mbitrate > self.mbitrate:
                        self._delayloop = self._delayloop + 1;
                    elif mbitrate < self.mbitrate and self._delayloop > 2:
                        self._delayloop = self._delayloop - 1;
                    start_time_of_100_packets = time.clock()

                    if i % 10000 == 9999:
                        print ("INFO Current bit rate = {} Mbit/s. Delay loop = {} Data sent = {} MB".format(mbitrate,self._delayloop,self.streamsize/1000000))


    def pcapstreamread(self,filename,dport=0,pkttype="inetx",dietpacket=True):
        '''This method will read a pcap file (argument). It will pick out the UDP packets and will
        validate all teh sequence numbers. It will also pull out the dropped packet event packets'''
        try:
            open(filename)
        except IOError:
            print 'Cannot read pcap file '

        pcapfile = pcap.pcap(filename)
        count = 0
        for ts, pkt in pcapfile:
            self.streamsize += len(pkt)
            count += 1
            # Is it an Ethernet packet?
            try:
                packet = dpkt.ethernet.Ethernet(pkt)

            except:
                continue
            # An IP packet. If not bail
            try:
                if packet.type == dpkt.ethernet.ETH_TYPE_IP:
                    # The data is an IP packet
                    ip = packet.data
                else:
                    continue
            except:
                continue

            # A UDP
            try:
                if ip.p == dpkt.ip.IP_PROTO_UDP:
                    # We have the udp packet!
                    udp = ip.data
                    srcip = socket.inet_ntoa(ip.src)
                    dstip = socket.inet_ntoa(ip.dst)
                else:
                    continue
            except:
                continue

            #print("Dst port =  {}".format(udp.dport))
            if pkttype == "inetx":
                pkt = iNetX()
            else:
                pkt = IENA()

            try:
                if (dport > 0 and udp.dport == dport) or (dport == 0):
                    pkt.packet = udp.data
                    pkt.unpack()
                    pkt.timestamp = ts
                    pkt.packetnumber = count
                    if dietpacket == True:
                        pkt.dietpkt()
                    pkt.srcip = srcip
                    pkt.dstip = dstip
                    pkt.dstport = udp.dport
                    self.packets.append(pkt)
            except:
                continue#
#
        self.length = len(self.packets)


    def extractcvsd(self,cvsdfile,srcip,ports=list()):
        for packet in self.packets:
            if packet.srcip != srcip:
                continue
            if len(ports) > 0:
                if not packet.dstport in ports:
                    continue

            packet.unpackpayload()
            cvsdf = open(cvsdfile,'ab')
            cvsdf.write(packet.payload)
            cvsdf.close()

    def validatestream(self,srcip,ports=list(),reporttime=0):
        '''Method to valudate the sequence numbers in a stream. It will record all missing sequence numbers
        for each stream and the first and last timestamp for the stream'''
        previous_sequence = dict()
        previous_gpslock = 2
        pktcount = 0
        previouspackettime = 0
        for packet in self.packets:
            pktcount = pktcount + 1
            if packet.srcip != srcip:
                continue
            if len(ports) > 0:
                if not packet.dstport in ports:
                    continue

            self.packetschecked += 1
            if self.firsttimestamp > packet.timestamp:
                self.firsttimestamp = packet.timestamp
            if self.lasttimestamp < packet.timestamp:
                self.lasttimestamp = packet.timestamp

            if packet.dstport == 9194:
                packet.unpackpayload()
                gpspacketstruct = struct.Struct('>HHHHHHHHHHH')
                packet_data = gpspacketstruct.unpack_from(packet.payload)
                gpslockint = int(math.floor(packet_data[10]/(2**15)))
                if gpslockint != previous_gpslock:
                    print "Packet Count = {} GPS Lock = {}".format(pktcount,gpslockint)
                    previous_gpslock = gpslockint


            if packet.dstport  == reporttime:
                packet.unpackpayload()

                pkttime = time.gmtime(packet.ptptimeseconds)
                if packet.ptptimeseconds != previouspackettime:
                    print "Packet PTP Time = {} sequence={} on streamid={} packet count={}".format(time.strftime("%H:%M:%S %d %b %Y",pkttime),packet.sequence,packet.streamid,pktcount)
                    if packet.ptptimeseconds - previouspackettime > 1:
                        print " -- ERROR -- Time jump of over 1 second"

                previouspackettime = packet.ptptimeseconds


            if packet.streamid not in previous_sequence :
                previous_sequence[packet.streamid] = packet.sequence
            elif((previous_sequence[packet.streamid] + 1) != packet.sequence):
                if (previous_sequence[packet.streamid] + 1) == pow(2,packet.sequencewidth) and packet.sequence == 0:
                    previous_sequence[packet.streamid] = packet.sequence
                else:
                    self.missedseqences[packet.streamid].append(previous_sequence[packet.streamid]+1)
                    self.missedcount += 1
                    self.missedtimes.append(packet.timestamp)
                    self.missedpacket[packet.packetnumber]=[previous_sequence[packet.streamid]+1,packet.sequence]
                    previous_sequence[packet.streamid] = packet.sequence
            else:
                previous_sequence[packet.streamid] = packet.sequence

    def streamdatarate(self):
        '''Return the data rate of the stream in Mbit/sec. To match reported values from wireshark'''
        return (self.streamsize * 8) / ((self.lasttimestamp - self.firsttimestamp)*1000*1000) # Mbit/se is 1000 not 1024 based on Wireshark

    def benchmark(self):
        '''Benchmark th computers performance. The data race of packet generation is  computer specific. This method
        will send a number of short streams and record the datarate achieved. Internally a delay loop is used to vary the rate
        and this method will calibrate that delay loop to a polynomial '''
        xlist = list()
        ylist = list()


        for i in range(22):
            # create a few streams and see what the data rates is
            mystream = iNetXStream(length=1000,payloadsize=self.payloadsize,parserblocks=self.parserblocks)
            mystream.dstip = self.dstip
            mystream.dstport = self.dstport
            mystream.srcport = self.srcport
            mystream.mbitrate = self.mbitrate
            mystream._delayloop = i*i*3
            t = timeit.Timer(mystream.buildandsend())
            tottime = t.timeit(1)
            ylist.append(mystream._delayloop) # the delay loop value used
            xlist.append(mystream.streamsize*8/(tottime*1000*1000)) # the data rate achieved
        # fit the data to a poly
        p = numpy.polyfit(xlist, ylist, deg=3)
        # store the coeff in the mbittodly list
        self._storembitdly(p.tolist())
        self._updatedlyloop()

    def _updatedlyloop(self):
        if self.mbitrate > 0:
            self._delayloop = int(self.mbitrate**3*self._mbittodly[0] + self.mbitrate**2*self._mbittodly[1] + self.mbitrate*self._mbittodly[2] + self._mbittodly[3])
        if self._delayloop < 0 :
            self._delayloop = 0

    def _storembitdly(self,mbitlist):
        self._mbittodly = mbitlist
        jsonencodedlist = json.dumps(mbitlist)
        f = open('.mbit.json', 'w')
        f.write(jsonencodedlist)
        f.close

    def _loadmbitdly(self):
        if os.path.exists('.mbit.json'):
            f = open('.mbit.json', 'r')
            self._mbittodly = json.load(f)
            f.close
        else:
            self._mbittodly = list()



__author__ = 'diarmuid'
import sys
sys.path.append("..")
import os

import unittest
import AcraNetwork.Pcap as pcap
import AcraNetwork.SimpleEthernet as SimpleEthernet
import struct

def getEthernetPacket(data=0xa):
    e = SimpleEthernet.Ethernet()
    e.srcmac = 0x001122334455
    e.dstmac = 0x998877665544
    e.type = SimpleEthernet.Ethernet.TYPE_IP
    e.payload = struct.pack("H",data)
    return e.pack()

class PcapBasicTest(unittest.TestCase):

    def test_missingfilename(self):
        self.assertRaises(TypeError,lambda: pcap.Pcap())

    def test_missingreadfile(self):
        self.assertRaises(IOError,lambda: pcap.Pcap("nofile.pcap"))

    def test_defaultMagicNumber(self):
        p = pcap.Pcap("_tmp.pcap",mode='w')
        self.assertEqual(p.magic,0xa1b2c3d4)

    def test_defaultVersionMaj(self):
        p = pcap.Pcap("_tmp.pcap",mode='w')
        self.assertEqual(p.versionmaj,2)

    def test_defaultVersionMin(self):
        p = pcap.Pcap("_tmp.pcap",mode='w')
        self.assertEqual(p.versionmin,4)

    def test_readTestFile(self):
        p = pcap.Pcap("test_input.pcap")
        p.readGlobalHeader()
        self.assertEqual(p.magic,0xa1b2c3d4)
        self.assertEqual(p.network,1)
        self.assertEqual(p.sigfigs,0)
        self.assertEqual(p.snaplen,262144)
        self.assertEqual(p.versionmaj,2)
        self.assertEqual(p.versionmin,4)
        self.assertEqual(p.zone,0)
        self.assertEqual(p.filesize,704)
        p.close()



    def test_readARecord(self):
        p = pcap.Pcap("test_input.pcap")
        p.readGlobalHeader()
        mypcaprecord = p.readAPacket()
        self.assertEqual(mypcaprecord.sec,1419678111)
        self.assertEqual(mypcaprecord.usec,811463)
        self.assertEqual(mypcaprecord.orig_len,70)
        self.assertEqual(mypcaprecord.incl_len,70)

    def test_writeARecord(self):
        p = pcap.Pcap("_tmp.pcap",mode='w')
        p.writeGlobalHeader()
        r = pcap.PcapRecord()
        r.setCurrentTime()
        r.packet = getEthernetPacket(0xa)
        p.writeARecord(r)
        p.close()
        p = pcap.Pcap("_tmp.pcap")
        p.readGlobalHeader()
        self.assertEqual(p.magic,0xa1b2c3d4)
        self.assertEqual(p.network,1)
        self.assertEqual(p.sigfigs,0)
        self.assertEqual(p.snaplen,65535)
        self.assertEqual(p.versionmaj,2)
        self.assertEqual(p.versionmin,4)
        self.assertEqual(p.zone,0)
        self.assertEqual(p.filesize,56)
        p.close()
        os.remove("_tmp.pcap")

    def test_appendARecord(self):
        p = pcap.Pcap("_tmp2.pcap",mode='w')
        p.writeGlobalHeader()
        r = pcap.PcapRecord()
        r.setCurrentTime()
        r.packet = getEthernetPacket(0xa)
        p.writeARecord(r)
        p.close()
        # Now try to append a record
        p = pcap.Pcap("_tmp2.pcap",mode='a')
        r.packet = getEthernetPacket(0xb)
        p.writeARecord(r)
        p.close()
        # Read back to verify
        p = pcap.Pcap("_tmp2.pcap")
        p.readGlobalHeader()
        self.assertEqual(p.filesize,88)
        rec1 = p.readAPacket()
        rec2 = p.readAPacket()
        e = SimpleEthernet.Ethernet()
        e.unpack(rec1.packet)
        self.assertEqual(e.payload,struct.pack("H",0xa))
        e.unpack(rec2.packet)
        self.assertEqual(e.payload,struct.pack("H",0xb))
        p.close()
        os.remove("_tmp2.pcap")



if __name__ == '__main__':
    unittest.main()




#-------------------------------------------------------------------------------
# Name:        pcap
# Purpose:     Class to pack and unpack pcap files
#
# Author:      DCollins
#
# Created:     19/12/2013
#
# Copyright 2014 Diarmuid Collins
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License
#    as published by the Free Software Foundation; either version 2
#    of the License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
import struct
import iNetX
import os
import SimpleEthernet
import time

class PcapRecord(object):
    '''Class that can be used to store one pcap record'''
    def __init__(self):
        self.sec = None
        """:type : int"""
        self.usec = None
        """:type : int"""
        self.incl_len = None
        """:type : int"""
        self.orig_len = None
        """:type : int"""
        self._packet = None
        """:type : str"""

    # Use a property on packet so that the length is triggered on it changing
    @property
    def packet(self):
        return self._packet

    @packet.setter
    def packet(self,p):
        self._packet = p
        self.incl_len = len(p)
        self.orig_len = self.incl_len

    def unpack(self,buf):
        '''Unpack the pcap header. Pass in a buffer containing the header
        :type buf: str
        '''
        if struct.calcsize(Pcap.RECORD_HEADER_FORMAT) > len(buf):
            raise ValueError("Header buffer too big to be a Pcap record header")
        (self.sec,self.usec,self.incl_len,self.orig_len) = struct.unpack(Pcap.RECORD_HEADER_FORMAT,buf)


    def pack(self):
        '''Pack a PcapRecord into a buffer'''
        if self.sec == None or self.usec == None or self.incl_len == None or self.orig_len == None or self.packet == None:
            raise ValueError("Cannot build record with undefined fields in the payload")
        return struct.pack(Pcap.RECORD_HEADER_FORMAT,self.sec,self.usec,self.incl_len,self.orig_len) + self.packet


    def setCurrentTime(self):
        currenttime = time.time()
        self.usec = int((currenttime%1)*1e6)
        self.sec = int(currenttime)


class Pcap(object):

    '''Pcap handling class. Supports basic handling of pcap files. Read and write.
    Assumption: Ethernet packets only supported'''

    GLOBAL_HEADER_FORMAT = '<IhhiIII'
    RECORD_HEADER_FORMAT = '<IIII'
    RECORD_HEADER_SIZE =  struct.calcsize(RECORD_HEADER_FORMAT)
    GLOBAL_HEADER_SIZE = struct.calcsize(GLOBAL_HEADER_FORMAT)

    def __init__(self,filename,**kwargs):
        '''Class for parsing pcap file.
        Create a new pcap object passing in the pcapfilename.
        Then call the other methods to parse the file

        :type filename: str
        :type forreading: bool
        '''

        self.filename = filename
        self.bytesread=0
        self.mode = kwargs.get('mode','r')

        if 'forreading' in kwargs:
            if kwargs['forreading']  == False:
                self.mode = 'w'
            raise DeprecationWarning("forreading is being deprecated and replaced with the mode argument")

        if self.mode == 'a':
            try:
                self.fopen = file(filename,'ab')
            except:
                raise IOError("Failed to open {} for appending".format(filename))

        elif self.mode == 'r':
            try:
                self.fopen = file(filename,'rb')
                self.filesize = os.path.getsize(filename)
            except Exception as e:
                raise IOError("Failed to open {} for reading {}".format(filename,e))
        else:
            self.fopen = file(filename,'wb')

        # Global header fields
        self.magic = 0xa1b2c3d4
        self.versionmaj = 2
        self.versionmin = 4
        self.zone = 0
        self.sigfigs = 0
        self.snaplen = 65535
        self.network  = 1 # Ethernet


    def readGlobalHeader(self):
        '''This method will read the pcap global header and unpack it.
        This should be the first method to call on reading a PCAP file
        '''

        header = self.fopen.read(Pcap.GLOBAL_HEADER_SIZE)
        self.bytesread += Pcap.GLOBAL_HEADER_SIZE
        (self.magic,self.versionmaj,self.versionmin,self.zone,self.sigfigs,self.snaplen,self.network) = struct.unpack(Pcap.GLOBAL_HEADER_FORMAT,header)

    def writeGlobalHeader(self):
        '''Write the global header to a new pcap file'''

        header = struct.pack(Pcap.GLOBAL_HEADER_FORMAT,self.magic,self.versionmaj,self.versionmin,self.zone,self.sigfigs,self.snaplen,self.network)
        self.fopen.write(header)

    def writeARecord(self,pcaprecord):
        '''Write the supplied pcaprecord to the pcap file
        :type pcaprecord: PcapRecord
        '''

        self.fopen.write(pcaprecord.pack())

    def close(self):
        '''Close the current pcap file'''
        self.fopen.close()

    def readAPacket(self):
        '''Method to read the next Packet in the pcap file
        :rtype: PcapRecord
        '''
        if self.bytesread >= self.filesize:
            raise IOError("Reached the end of the packet")

        # read the pcap header to a new object
        pcaprecord = PcapRecord()
        pcaprecord.unpack(self.fopen.read(Pcap.RECORD_HEADER_SIZE))
        try:
            pcaprecord.packet = self.fopen.read(pcaprecord.incl_len)
        except:
            raise IOError("Failed to read {} bytes from pcap file".format(pcaprecord.incl_len))
        self.bytesread += pcaprecord.incl_len+Pcap.RECORD_HEADER_SIZE

        return pcaprecord





    #-------------------------------------------------
    # Old, deprecated methods that should not be used
    # and I'm not going to test for them
    #-------------------------------------------------
    def _readNextiNetXPacket(self):
        """This method will read the next iNetX packet one by one. It will raise an IOError when it hits
        the end of the file and n NoImplementedError when it hits packets that are not iNetx. It returns
        the ip, udp and inetx packets"""
        # already at the end of the file
        if self.bytesread >= self.filesize:
            raise IOError

        # otherwise pull our the PCAP header size. Should really push headersize to the object self
        pcapheader = self.fopen.read(Pcap.RECORD_HEADER_SIZE)
        (sec,usec,incl_len,orig_len) = struct.unpack(Pcap.RECORD_HEADER_FORMAT,pcapheader)

        # now we have a packet then lets deconstruct it. first read in the ethernet header and unpack it
        eth_header = SimpleEthernet.Ethernet(self.fopen.read(SimpleEthernet.Ethernet.HEADERLEN))

        # This is the most portable but as a quick hack I'm going
        # to ignore smoe packets. All non-IP packets for starter
        if eth_header.type != 2048:
            throwaway=self.fopen.read(orig_len-SimpleEthernet.Ethernet.HEADERLEN)
            self.bytesread += (orig_len+Pcap.RECORD_HEADER_SIZE)
            raise NotImplementedError

        # pull apart the other headers. Again I'm assuming all the IP packets
        # are udp packets
        ip_header = SimpleEthernet.IP(self.fopen.read(SimpleEthernet.IP.HEADERLEN))
        udp_header = SimpleEthernet.UDP(self.fopen.read(SimpleEthernet.UDP.HEADERLEN))

        # Lets create an iNetx packet then test if we are really dealing with an
        # inetx packet. Do this by testing the first 4 bytes which should be the control word
        CONTROLWORD_LEN=4
        CONTROLWORD_STRING = struct.pack('>I',0x11000000)
        inetx_packet = iNetX.iNetX()
        inetx_packet.packet = self.fopen.read(CONTROLWORD_LEN)

        # So are the first 4 bytes the control word
        if inetx_packet.packet != CONTROLWORD_STRING:
            # throw away the rest of the packet and bail out with an exception
            throwaway=self.fopen.read(orig_len-SimpleEthernet.Ethernet.HEADERLEN-SimpleEthernet.IP.HEADERLEN-SimpleEthernet.UDP.HEADERLEN-CONTROLWORD_LEN)
            self.bytesread += (orig_len+headersize)
            raise NotImplementedError

        # So we have an inetx packet, read in the rest of the pcap packet and unpack it as an inetX packet
        inetx_packet.packet += self.fopen.read(orig_len-SimpleEthernet.Ethernet.HEADERLEN-SimpleEthernet.IP.HEADERLEN-SimpleEthernet.UDP.HEADERLEN-CONTROLWORD_LEN)
        inetx_packet.unpack(inetx_packet.packet)

        self.bytesread += (headersize + orig_len)
        # return the interesting packet headers, the packet time and the temperature if it exists
        return (inetx_packet,ip_header,udp_header,sec,bcu_temperature)


    def _readNextUDPPacket(self):
        """This method will read the next UDP packet in the pcap file one by one and returns the udp,eth and ip packets"""
        # already at the end of the file
        if self.bytesread >= self.filesize:
            raise IOError

        # otherwise pull our the PCAP header size. Should really push headersize to the object self
        headersize=struct.calcsize(Pcap.RECORD_HEADER_FORMAT)
        pcapheader = self.fopen.read(headersize)
        self.bytesread +=headersize
        (sec,usec,incl_len,orig_len) = struct.unpack(Pcap.RECORD_HEADER_FORMAT,pcapheader)

        # now we have a packet then lets deconstruct it. first read in the packet
        eth_pkt = SimpleEthernet.Ethernet(self.fopen.read(orig_len))
        self.bytesread +=orig_len

        # This is the most portable but as a quick hack I'm going
        # to ignore smoe packets. All non-IP packets for starter
        if eth_pkt.type != 2048:
            raise NotImplementedError

        # pull apart the other headers. Again I'm assuming all the IP packets
        # are udp packets
        ip_pkt = SimpleEthernet.IP(eth_pkt.payload)
        # Only parse udp
        if ip_pkt.protocol != SimpleEthernet.IP.PROTOCOLS['UDP']:
            raise NotImplementedError

        udp_pkt = SimpleEthernet.UDP(ip_pkt.payload)

        # return the interesting packet headers, the packet time and the temperature if it exists
        return (eth_pkt,ip_pkt,udp_pkt)


__author__ = 'diarmuid'
import sys
sys.path.append("..")
import os

import unittest
import AcraNetwork.SimpleEthernet as SimpleEthernet
import AcraNetwork.Pcap as pcap
import struct

class SimpleEthernetTest(unittest.TestCase):

    ######################
    # Ethernet
    ######################
    def test_DefaultEthernet(self):
        e = SimpleEthernet.Ethernet()
        self.assertEqual(e.dstmac,None)
        self.assertEqual(e.srcmac,None)
        self.assertEqual(e.type,None)
        self.assertEqual(e.payload,None)

    def test_basicEthernet(self):
        '''Create an ethernet frame, then unpack it to a new object'''
        e = SimpleEthernet.Ethernet()
        e.srcmac = 0x001122334455
        e.dstmac = 0x998877665544
        e.type = SimpleEthernet.Ethernet.TYPE_IP
        e.payload = struct.pack("H",0xa)
        ethbuf = e.pack()

        e2  = SimpleEthernet.Ethernet()
        e2.unpack(ethbuf)

        self.assertEqual(e2.dstmac,0x998877665544)
        self.assertEqual(e2.type,SimpleEthernet.Ethernet.TYPE_IP)
        self.assertEqual(e2.srcmac,0x001122334455)

    def test_buildEmptyEthernet(self):
        '''Try and create an empty ethernet frame'''
        e = SimpleEthernet.Ethernet()
        self.assertRaises(ValueError,lambda: e.pack())


    ######################
    # IP
    ######################
    def test_defaultIP(self):
        i = SimpleEthernet.IP()
        self.assertRaises(ValueError, lambda : i.pack())

    def test_basicIP(self):
        i = SimpleEthernet.IP()
        i.dstip = "235.0.0.1"
        i.srcip = "192.168.1.1"
        i.payload = struct.pack(">H",0xa5)
        ippayload = i.pack()

        i2 = SimpleEthernet.IP()
        i2.unpack(ippayload)
        self.assertEqual(i2.srcip,"192.168.1.1")
        self.assertEqual(i2.dstip,"235.0.0.1")
        self.assertEqual(i2.payload,struct.pack(">H",0xa5))

    def test_unpackIPShort(self):
        i = SimpleEthernet.IP()
        dummypayload = struct.pack('H',0xa5)
        self.assertRaises(ValueError, lambda : i.unpack(dummypayload))

    ######################
    # UDP
    ######################

    def test_defaultUDP(self):
        u = SimpleEthernet.UDP()
        self.assertRaises(ValueError,lambda :u.pack())

    def test_basicUDP(self):
        u = SimpleEthernet.UDP()
        u.dstport = 5500
        u.srcport = 4400
        u.payload = struct.pack('B',0x5)
        mypacket = u.pack()
        self.assertEqual(mypacket,struct.pack('>HHHHB',4400,5500,9,0,0x5))

    def test_unpackUDPShort(self):
        u = SimpleEthernet.UDP()
        dymmypayload =  struct.pack('H',0xa5)
        self.assertRaises(ValueError,lambda : u.unpack(dymmypayload))

    ######################
    # Read a complete pcap file
    ######################
    def test_readUDP(self):
        p = pcap.Pcap("test_input.pcap")
        p.readGlobalHeader()
        mypcaprecord = p.readAPacket()
        e = SimpleEthernet.Ethernet()
        e.unpack(mypcaprecord.packet)
        self.assertEqual(e.srcmac,0x0018f8b84454)
        self.assertEqual(e.dstmac,0xe0f847259336)
        self.assertEqual(e.type,0x0800)

        i = SimpleEthernet.IP()
        i.unpack(e.payload)
        self.assertEqual(i.dstip,"192.168.1.110")
        self.assertEqual(i.srcip,"213.199.179.165")
        self.assertEqual(i.protocol,0x6)
        self.assertEqual(i.ttl,48)
        self.assertEqual(i.flags,0x2)
        self.assertEqual(i.id,0x4795)
        self.assertEqual(i.len,56)

if __name__ == '__main__':
    unittest.main()



#-------------------------------------------------------------------------------
# Name:        iNetX
# Purpose:     Class to construct and de construct iNetx Packets
#
# Author:      DCollins
#
# Created:     19/12/2013
#
# Copyright 2014 Diarmuid Collins
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License
#    as published by the Free Software Foundation; either version 2
#    of the License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


import struct
import datetime

NAL_HEADER = 0x00000001
NAL_HEADER_LEN = 4
NAL_TYPES = { "SEI":6, "Unspecified" : 0, "Coded IDR" : 5, "SPS" : 7, "PPS" : 8, "AUD" : 9,
              "EOS" : 10, "DPS" : 16}
# Invert it to go from integer to more useful name
NAL_TYPES_INV = dict((v, k) for k, v in NAL_TYPES.iteritems())
SEI_UNREG_DATA = 5


class MPEGTS():
    '''
    This class handles MPEG Transport Streams.
    https://en.wikipedia.org/wiki/MPEG_transport_stream

    Each transport stream contains 188 byte packets.
    These packets contain either video, audio or metadata information

    '''

    def __init__(self):
        '''
        Each MPEGTS contains an array of MPEGTS Packets
        '''
        self.previouscounter = {}
        self.discontinuity ={}
        self.blocks = []
        self.contunityerror = False
        self.invalidsync = False
        self.invalidsyncblock = list()

    def unpack(self,buf):
        '''
        This method will convert a buffer of bytes into an array of MPEG TS packets
        :param buf:
        :return:
        '''
        remainingbytes = 0
        prevcount = self.previouscounter
        block_count = 0
        while remainingbytes < len(buf):
            MpegBlock = MPEGPacket()
            MpegBlock.unpack(buf[remainingbytes:remainingbytes+188])
            block_count += 1
            self.blocks.append(MpegBlock)
            if MpegBlock.invalidsync == True:
                self.invalidsync = True
                self.invalidsyncblock.append(block_count)
            remainingbytes += 188
            if  not MpegBlock.pid in prevcount:
                prevcount[MpegBlock.pid] = MpegBlock.continuitycounter
            elif ((prevcount[MpegBlock.pid]+1) % 16) != MpegBlock.continuitycounter:
                self.contunityerror = True
                self.discontinuity[MpegBlock.pid] = (prevcount[MpegBlock.pid],MpegBlock.continuitycounter)
                prevcount[MpegBlock.pid] = MpegBlock.continuitycounter
            else:
                prevcount[MpegBlock.pid] = MpegBlock.continuitycounter

        self.previouscounter = prevcount

    def NumberOfBlocks(self):
        return len(self.blocks)

    def FirstCount(self):
        return self.blocks[0].continuitycounter

    def LastCount(self):
        return self.blocks[self.NumberOfBlocks()-1].continuitycounter


class MPEGPacket():
    '''
    The MPEGPacket is the elementary unit in an MPEG Transport Stream
    It contains an header, in which there's a sync word, continuity counter, and a payload
    '''
    def __init__(self):
        self.packetstrut = struct.Struct('>BHB')
        self.sync = None
        self.pid = None
        self.continuitycounter = None
        self.invalidsync = False
        self.payload = ""


    def unpack(self,buf):
        '''
        Converts a buffer into an MPEGTS packet
        :param buf:
        :return:
        '''
        (self.syncbyte,pid_full,counter_full)  = self.packetstrut.unpack_from(buf)
        if self.syncbyte != 0x47:
            self.invalidsync = True

        self.pid = pid_full % 8192
        self.continuitycounter = counter_full % 16
        self.payload = buf[struct.calcsize(">BHB"):]


class H264(object):
    '''
    This class will handle H.264 payload. It can convert a buffer of bytes into an array
    of NALs(https://en.wikipedia.org/wiki/Network_Abstraction_Layer)
    The NALs contain different data, based on their types.
    '''

    def __init__(self):
        self.nals = []

    def unpack(self,buf):
        """
        Unpack a buffer into the NALs
        :param buf:
        :return:
        """
        nal_hdr = struct.pack(">L",NAL_HEADER)
        offsets = string_matching_boyer_moore_horspool(buf,nal_hdr)

        for idx,offset in enumerate(offsets):
            if idx == len(offsets)-1:
                nal_buf = buf[offset:]
            else:
                nal_buf = buf[offset:(offsets[idx+1])]
            nal = NAL()
            nal.unpack(nal_buf)
            nal.offset = offset
            self.nals.append(nal)

        return True

class NAL(object):
    '''
    The NAL can be split into the various types of NALs.
    '''
    def __init__(self):
        self.type = None
        self.size = None
        self.sei = None
        self.offset = None

    def unpack(self,buf):
        # First 4 bytes are the NAL_HEADER, then forbidden + type
        (self.type,) = struct.unpack(">B",buf[NAL_HEADER_LEN])
        self.type = self.type & 0x1F
        if self.type == NAL_TYPES["SEI"]:
            sei = STANAG4609_SEI()
            sei.unpack(buf[(NAL_HEADER_LEN+1):])
            self.sei = sei


class STANAG4609_SEI(object):
    '''
    Handle the SEI NAL and more specifically this will handle SEIs defined in 3.14.3.5 of the STANAG standard
    http://www.gwg.nga.mil/misb/docs/nato_docs/STANAG_4609_Ed3.pdf
    '''
    def __init__(self):
        self.payloadtype = None
        self.payloadsize = None
        self.unregdata = False
        self.status = None
        self.seconds = None
        self.microseconds = None
        self.nanoseconds = None
        self.time = None
        self.stanag = False

    def unpack(self,buf):
        (self.payloadtype,self.payloadsize) = struct.unpack(">BB",buf[0:2])
        if self.payloadtype == SEI_UNREG_DATA:
            self.unregdata = True
            (sig1,sig2,self.status,ms1,_fix1,ms2,_fix2,ms3,_fix3,ms4,) = struct.unpack_from(">QQBHBHBHBH",buf[2:])
            # combine the time fields (cf  http://www.gwg.nga.mil/misb/docs/nato_docs/STANAG_4609_Ed3.pdf 3.14.3.4 )
            # Verify the signature and if it's good then convert to a time
            if sig1 == 0x4d4953506d696372 and sig2 == 0x6f73656374696d65 and \
                            _fix1 == 0xff and _fix2 == 0xff and _fix3 == 0xff:
                useconds = (ms1<<48)+(ms2<<32)+(ms3<<16)+ms4
                self.seconds = float(useconds)/1.0e6
                self.nanoseconds = (ms3<<16)+ms4
                self.time = datetime.datetime.fromtimestamp(self.seconds)
                self.stanag = True


def string_matching_boyer_moore_horspool(text='', pattern=''):
    """Returns positions where pattern is found in text.
    O(n)
    Performance: ord() is slow so we shouldn't use it here
    Example: text = 'ababbababa', pattern = 'aba'
         string_matching_boyer_moore_horspool(text, pattern) returns [0, 5, 7]
    @param text text to search inside
    @param pattern string to search for
    @return list containing offsets (shifts) where pattern is found inside text
    """
    m = len(pattern)
    n = len(text)
    offsets = []
    if m > n:
        return offsets
    skip = []
    for k in range(256):
        skip.append(m)
    for k in range(m-1):
        skip[ord(pattern[k])] = m - k - 1
    skip = tuple(skip)
    k = m - 1
    while k < n:
        j = m - 1
        i = k
        while j >= 0 and text[i] == pattern[j]:
            j -= 1
            i -= 1
        if j == -1:
            offsets.append(i + 1)
        k += skip[ord(text[k])]

    return offsets



#-------------------------------------------------------------------------------
# Name:        module1
# Purpose:
#
# Author:      Diarmuid
#
# Created:     16/12/2013
# Copyright:   (c) SPACE 2013
# Licence:     <your licence>
#-------------------------------------------------------------------------------
import sys
sys.path.append("..")

import socket

import AcraNetwork.iNetX as inetx
import AcraNetwork.McastSocket as McastSocket
udp_port = 5555

#------------------------------------------------------------
# Setup a socket to recieve all traffic
#------------------------------------------------------------
try:
    recv_socket = McastSocket.McastSocket(local_port=udp_port, reuse=1)
    recv_socket.mcast_add('235.0.0.2', '0.0.0.0')
    recv_socket.settimeout(10)
except:
    print "Can't bind to socket {}".format(udp_port)
    exit()

packet_count = 1
while True:
    try:
        data, addr = recv_socket.recvfrom(2048) # buffer size is 1500 bytes
    except socket.timeout:
        print "timeout on socket"
        exit()

    (udpsrcport,srcipaddr) = addr
    avionics_packet = inetx.iNetX()
    data_len = len(data)
    try:
        avionics_packet.unpack(data)
    except ValueError:
        # This isn't an inetx packet
        packet_count += 1
        continue
    else:
        print "Packet withStread ID = {:0X}".format(avionics_packet.streamid)


#-------------------------------------------------------------------------------
# Name:        
# Purpose:     
#
# Author:      
#
# Created:     
#
# Copyright 2014 Diarmuid Collins
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License
#    as published by the Free Software Foundation; either version 2
#    of the License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
__author__ = '$USER'

import sys
sys.path.append("..")

import unittest
import datetime
import time
import AcraNetwork.IENA as iena
import AcraNetwork.SimpleEthernet as SimpleEthernet
import AcraNetwork.Pcap as pcap

import struct

class IENATest(unittest.TestCase):

    def test_defaultIENA(self):
        '''Check the defaults for the IENA packet'''
        i = iena.IENA()
        self.assertEqual(i.key,None)
        self.assertEqual(i.size,None)
        self.assertEqual(i.keystatus,None)
        self.assertEqual(i.status,None)
        self.assertEqual(i.sequence,None)
        self.assertEqual(i.endfield,0xdead)
        self.assertEqual(i.payload,None)

    def test_basicIENA(self):
        '''Build a very basic packet and check the packed result'''
        i = iena.IENA()
        i.key = 1
        i.keystatus = 2
        i.status = 3
        i.sequence = 10
        i.setPacketTime(time.mktime(datetime.datetime(datetime.datetime.today().year, 1, 2, 0, 0, 0,0).timetuple()),0)
        i.payload = struct.pack('H',0x5)
        # size = 9. Time = midnight Jan 02 = 86400s 0us = 0x14dd760000
        expected_payload = struct.pack(iena.IENA.IENA_HEADER_FORMAT,1,9,0x14,0x1dd76000,2,3,10) + struct.pack('H',0x5) + struct.pack('>H',0xdead)
        self.assertEqual(i.pack(),expected_payload)

    def test_unpackIENA(self):
        pass

    def test_unpackIEANFromPcap(self):
        '''Read all the IENA packets in a pcap file and check each field'''
        p = pcap.Pcap("iena_test.pcap")
        p.readGlobalHeader()
        sequencenum = 195
        exptime = 0x1d102f800
        while True:
            # Loop through the pcap file reading one packet at a time
            try:
                mypcaprecord = p.readAPacket()
            except IOError:
                # End of file reached
                break
            e = SimpleEthernet.Ethernet()
            e.unpack(mypcaprecord.packet)
            ip =  SimpleEthernet.IP()
            ip.unpack(e.payload)
            u = SimpleEthernet.UDP()
            u.unpack(ip.payload)
            # Now I have a payload that will be an iena packet
            i = iena.IENA()
            i.unpack(u.payload)
            self.assertEquals(i.key,0x1a)
            self.assertEquals(i.size,24)
            self.assertEquals(i.status,0)
            self.assertEquals(i.keystatus,0)
            self.assertEquals(i.sequence,sequencenum)
            sequencenum += 1
            self.assertEqual(i.timeusec,exptime)
            exptime += 0x186a0 # The timestamp increments by a fixed number of microseconds
            self.assertEquals(i.endfield,0xdead)



if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

from distutils.core import setup
import shutil

try:
    from pypandoc import convert
    output = convert('README.md', 'rst', outputfile="README.txt")
except ImportError:
    print("warning: pypandoc module not found, could not convert Markdown to RST")
    shutil.copyfile("README.md","README.txt")

exec(open('AcraNetwork/__version__.py').read())

setup(name='AcraNetwork',
      version=__version__,
      description='Classes and utilities to support Flight Test Instrumentation Ethernet networks',
      author='Diarmuid Collins',
      author_email='dcollins@curtisswright.com',
      url='https://github.com/diarmuidcwc/AcraNetwork',
      packages=['AcraNetwork'],
      long_description=open('README.txt', 'rt').read(),
      classifiers =['Programming Language :: Python',
                    'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
                    'Operating System :: OS Independent',
                    'Development Status :: 3 - Alpha',
                    'Intended Audience :: Developers',
                    'Topic :: Software Development :: Libraries :: Python Modules',
                    ],
     )



import AcraNetwork.Pcap as pcap
import AcraNetwork.iNetX as inetx
import AcraNetwork.MPEGTS as mpegts
import AcraNetwork.SimpleEthernet as eth
from AcraNetwork.MPEGTS import H264
import datetime

# This script shows how to parse either a pcap file or a TS file into the constituent
# NALS and finds the unique STANAG SEI User data with the timestamp

def pcap_to_ts(pcapfile,ts_file,udp_port=8010):
    '''
    Convert a pcap file to a TS file by extracting all data from a specified port
    :param mpegfile: str
    :param ts_file: str
    :param udp_port: int
    :return:
    '''

    mpegpcap = pcap.Pcap(pcapfile, mode='r')
    mpegpcap.readGlobalHeader()

    ts = open(ts_file, mode='wb')
    mpeghex = ""
    rec_count = 0

    while not ts.closed:
        try:
            rec = mpegpcap.readAPacket()
        except:
            ts.close()
        else:
            try:
                e = eth.Ethernet()
                e.unpack(rec.packet)
                i = eth.IP()
                i.unpack(e.payload)
                u = eth.UDP()
                u.unpack(i.payload)
                if u.dstport == udp_port:
                    rec_count += 1
                    inet = inetx.iNetX()
                    inet.unpack(u.payload)
                    mpegtspackets = mpegts.MPEGTS()
                    mpegtspackets.unpack(inet.payload)
                    for packet in mpegtspackets.blocks:
                        mpeghex += packet.payload
                    ts.write(inet.payload)
            except:
                continue


def parse_ts_file(tsfile):

    ts_file = open(tsfile,mode='rb')
    h264_data = H264()
    h264_data.unpack(ts_file.read())

    nal_counts = {}
    timestamp_count = 0
    for nal in h264_data.nals:

        if not nal.type in nal_counts:
            nal_counts[nal.type] = 1
        else:
            nal_counts[nal.type] += 1

        if nal.type == mpegts.NAL_TYPES["SEI"]:
            if nal.sei.unregdata:
                print "Timestamp={} byte offset={} count ={}".format(datetime.datetime.strftime(nal.sei.time,"%d %b %Y %H:%M:%S.%f"),nal.offset,timestamp_count)
                timestamp_count += 1

        else:
            pass

    print "\n----- SUMMARY -----"
    for type in nal_counts:
        print "{} {} NALs in input".format(nal_counts[type],mpegts.NAL_TYPES_INV[type])
    print "{} STANAG Timestamps".format(timestamp_count)

def main():
    # Read in a TS file and print out some useful information
    parse_ts_file("../test/stanag_sample.ts")


if __name__ == "__main__":
    main()


# -------------------------------------------------------------------------------
# Name:
# Purpose:
#
# Author:
#
# Created:
#
# Copyright 2014 Diarmuid Collins
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License
#    as published by the Free Software Foundation; either version 2
#    of the License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import sys
sys.path.append("..")
import time
import struct
import AcraNetwork.IENA as iena
import AcraNetwork.iNetX as inetx
import AcraNetwork.McastSocket as mcast


# simple application that tests building and sending of iena and inetx packets

UDP_IP = "235.0.0.1"
UDP_PORT = 5005

print "UDP target IP:", UDP_IP
print "UDP target port:", UDP_PORT


sock = mcast.McastSocket()
#sock.mcast_add(UDP_IP)

# Fixed payload for both
payload = struct.pack(">L",5)


# Create an inetx packet
myinetx = inetx.iNetX()
myinetx.inetxcontrol = inetx.iNetX.DEF_CONTROL_WORD
myinetx.pif = 0
myinetx.streamid = 0xdc
myinetx.sequence = 0
myinetx.payload = payload

# Create an iena packet
myiena = iena.IENA()
myiena.key = 0xdc
myiena.keystatus = 0
myiena.endfield = 0xbeef
myiena.sequence = 0
myiena.payload = payload
myiena.status = 0



while True:

    currenttime = int(time.time())

    myiena.sequence += 1
    myiena.setPacketTime(currenttime)
    #sock.sendto(myiena.pack(), (UDP_IP, UDP_PORT))
    #print "iena sent"

    myinetx.sequence += 1
    myinetx.setPacketTime(currenttime)
    sock.sendto(myinetx.pack(), (UDP_IP, UDP_PORT+1))
    print "inetx sent"

    time.sleep(2)



#-------------------------------------------------------------------------------
# Name:        pcap_to_ascii.py
# Purpose:
#
# Author:      Diarmuid
#
# Created:     18/03/2014
#
# Copyright 2014 Diarmuid Collins
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# About:       This is a simple test script that loops through a pcap file
#               Finds all iNetX packets and dumps the ASCII representation of the
#               data to the output
#-------------------------------------------------------------------------------
import sys
sys.path.append("..")

import socket,os,struct,sys
import argparse
import datetime, time


import AcraNetwork.iNetX as inetx
import AcraNetwork.Pcap as pcap
import AcraNetwork.SimpleEthernet as SimpleEthernet
import AcraNetwork.ParserAligned as ParserAligned

def main():

    #----------------------------------
    # Setup the command line parser
    #----------------------------------
    parser = argparse.ArgumentParser(description='Dump out the payload of iNetX packets as ASCII representations')
    parser.add_argument('--pcap',  required=True, action='append',  help='The input pcap file(s)')
    parser.add_argument('--hex',  required=False, action='store_true', default=False,  help='Print the hex representation not the ASCII coded version')
    parser.add_argument('--outdir',  required=False, default="out", help='Name of output directory. Default is out')
    args = parser.parse_args()

    #------------------------------------------------------------
    # Now read the input.
    #------------------------------------------------------------
    # The input will take multiple pcap files and loop through each

    # Keep a track of the position in the line for each streamID
    output_byte_count ={}

    for pcapfilename in args.pcap:
        try:
            pcapfile = pcap.Pcap(pcapfilename)
        except IOError:
            print "ERROR: File {} not found".format(pcapfilename)
            exit()


        if not os.path.exists(args.outdir):
            os.mkdir(args.outdir)

        pcapfile.readGlobalHeader()
        while True:
            try:

                # So we loop through the file one packet at a time. This will eventually return an
                # exception at the end of file so handle that when it occurs


                pcaprecord = pcapfile.readAPacket()
                eth = SimpleEthernet.Ethernet()
                eth.unpack(pcaprecord.packet)
                ip = SimpleEthernet.IP()
                ip.unpack(eth.payload)
                udp_packet = SimpleEthernet.UDP()
                udp_packet.unpack(ip.payload)
                (ctrl_word,) = struct.unpack('>I',udp_packet.payload[:4])
                if ctrl_word == 0x11000000:
                    inetx_packet = inetx.iNetX()
                    # Unpack the udp payload as an iNetx packet
                    inetx_packet.unpack(udp_packet.payload)
                    # Do we want to dump out an ascii or hex output
                    if args.hex == True:
                        prefix = "hex"
                    else:
                        prefix = "ascii"

                    # Create an output  file per streamID and open it
                    output_file_name = "{}/{}_{:08X}.txt".format(args.outdir,prefix,inetx_packet.streamid)
                    # NB: We are appending to the file here so if you have existing files in the directory then it will be appended
                    output_file = open(output_file_name,'a')

                    # Start the byte count per streamID
                    if  not output_byte_count.has_key(inetx_packet.streamid):
                        output_byte_count[inetx_packet.streamid] = 1

                    # Go thorough each byte in the payload. Not particularly efficient
                    for byte in inetx_packet.payload:
                        # Unpack the payload as an unsigned integer
                        (byte_in_ascii,) =struct.unpack('B',byte)

                        # Write the output depending on what you want
                        if args.hex == True:
                            output_file.write("{:02X} ".format(byte_in_ascii))
                        else:
                            # Only some ASCII codes are printable so don't print out
                            # the non printable ones. Emulate the wireshark method of printing a period
                            if byte_in_ascii < 31 or byte_in_ascii > 126:
                                printable_string = "."
                            else:
                                printable_string = chr(byte_in_ascii)

                            output_file.write("{}".format(printable_string))

                        # Create a new line after 16 bytes for readability
                        if (output_byte_count[inetx_packet.streamid] % 16 == 0):
                            output_file.write('\n')
                        output_byte_count[inetx_packet.streamid] += 1



            except NotImplementedError:
                # We received a packet that we don't care about. So skip silently
                pass

            except IOError:
                # We are at the end of the file so lets jump to the next file
                print ( "End of {} reached.".format(pcapfilename))
                break

        print "Output files created in {} directory".format(args.outdir)



if __name__ == '__main__':
    main()
